-------------------------------------------------------------------------------
-- $Id$
-------------------------------------------------------------------------------
-- char_gen_rom.vhdl
-------------------------------------------------------------------------------
--
--                  ****************************
--                  ** Copyright Xilinx, Inc. **
--                  ** All rights reserved.   **
--                  ****************************
--
-------------------------------------------------------------------------------
-- Filename:        char_gen_rom.vhdl
--
-- Description:     
--                  
--   This module creates a 2k x 8 ROM that is used for video character generation.
--   The ROM contents are also defined in the CHAR_GEN_ROM.ucf file.
--
--   The charcter generator defines a 8 x 8 block of pixels used for ASCII characters,
--   line graphics and Xilinx logo. The least significant 3 address bits define the line
--   within the character block and the most significant 8 bits define the specific character.
--              
--   If the user wishes to extend the character set then the "INIT_20 - INIT_3F" values must be
--   defined.
--
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--              char_gen_rom.vhdl
--
-------------------------------------------------------------------------------
-- Author:          goran
-- Revision:        $Revision$
-- Date:            $Date$
--
-- History:
--   rickb  2002-01-18    First Version
--   goran  2002-01-18    VHDL Version
--
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

library unisim;
use unisim.all;

entity char_gen_rom is
  
  port (
    -- OPB read and write port
    Clk           : in  std_logic;
    Address       : in  std_logic_vector(10 downto 0);
    Read_Data     : out std_logic_vector(7 downto 0);
    Write_Data    : in  std_logic_vector(7 downto 0);
    Write         : in  std_logic;
    -- Pixel read port
    Pixel_Clock   : in  std_logic;
    Pixel_Address : in  std_logic_vector(10 downto 0);
    Pixel_Data    : out std_logic_vector(7 downto 0));

end entity char_gen_rom;

architecture IMP of char_gen_rom is

  component RAMB16_S9_S9 is
    port (
      DIA   : in  std_logic_vector (7 downto 0);
      DIB   : in  std_logic_vector (7 downto 0);
      DIPA  : in  std_logic_vector (0 downto 0);
      DIPB  : in  std_logic_vector (0 downto 0);
      ENA   : in  std_ulogic;
      ENB   : in  std_ulogic;
      WEA   : in  std_ulogic;
      WEB   : in  std_ulogic;
      SSRA  : in  std_ulogic;
      SSRB  : in  std_ulogic;
      CLKA  : in  std_ulogic;
      CLKB  : in  std_ulogic;
      ADDRA : in  std_logic_vector (10 downto 0);
      ADDRB : in  std_logic_vector (10 downto 0);
      DOA   : out std_logic_vector (7 downto 0);
      DOB   : out std_logic_vector (7 downto 0);
      DOPA  : out std_logic_vector (0 downto 0);
      DOPB  : out std_logic_vector (0 downto 0));
  end component RAMB16_S9_S9;

  attribute INIT_00 : string;
  attribute INIT_01 : string;
  attribute INIT_02 : string;
  attribute INIT_03 : string;
  attribute INIT_04 : string;
  attribute INIT_05 : string;
  attribute INIT_06 : string;
  attribute INIT_07 : string;
  attribute INIT_08 : string;
  attribute INIT_09 : string;
  attribute INIT_0A : string;
  attribute INIT_0B : string;
  attribute INIT_0C : string;
  attribute INIT_0D : string;
  attribute INIT_0E : string;
  attribute INIT_0F : string;
  attribute INIT_10 : string;
  attribute INIT_11 : string;
  attribute INIT_12 : string;
  attribute INIT_13 : string;
  attribute INIT_14 : string;
  attribute INIT_15 : string;
  attribute INIT_16 : string;
  attribute INIT_17 : string;
  attribute INIT_18 : string;
  attribute INIT_19 : string;
  attribute INIT_1A : string;
  attribute INIT_1B : string;
  attribute INIT_1C : string;
  attribute INIT_1D : string;
  attribute INIT_1E : string;
  attribute INIT_1F : string;
  attribute INIT_20 : string;
  attribute INIT_21 : string;
  attribute INIT_22 : string;
  attribute INIT_23 : string;
  attribute INIT_24 : string;
  attribute INIT_25 : string;
  attribute INIT_26 : string;
  attribute INIT_27 : string;
  attribute INIT_28 : string;
  attribute INIT_29 : string;
  attribute INIT_2A : string;
  attribute INIT_2B : string;
  attribute INIT_2C : string;
  attribute INIT_2D : string;
  attribute INIT_2E : string;
  attribute INIT_2F : string;
  attribute INIT_30 : string;
  attribute INIT_31 : string;
  attribute INIT_32 : string;
  attribute INIT_33 : string;
  attribute INIT_34 : string;
  attribute INIT_35 : string;
  attribute INIT_36 : string;
  attribute INIT_37 : string;
  attribute INIT_38 : string;
  attribute INIT_39 : string;
  attribute INIT_3A : string;
  attribute INIT_3B : string;
  attribute INIT_3C : string;
  attribute INIT_3D : string;
  attribute INIT_3E : string;
  attribute INIT_3F : string;

  attribute INIT_00 of CHARACTER_GEN_RAM : label is "1F3E7CF87C3E1F07C0000000040E1F3F071F3E7CF87C3E1FFFFFFFFFFFFFFFFF";
  attribute INIT_01 of CHARACTER_GEN_RAM : label is "FF8080808080808001010101010101FF80808080808080FF3F1F0E04000000C0";
  attribute INIT_02 of CHARACTER_GEN_RAM : label is "01010101FF01010110101010101010FF10101010FF101010FF01010101010101";
  attribute INIT_03 of CHARACTER_GEN_RAM : label is "00102040FF40201000080402FF02040880808080FF808080FF10101010101010";
  attribute INIT_04 of CHARACTER_GEN_RAM : label is "00000000FF0000000000000000FF0000000000000000FF0000000000000000FF";
  attribute INIT_05 of CHARACTER_GEN_RAM : label is "FF0000000000000000FF0000000000000000FF0000000000000000FF00000000";
  attribute INIT_06 of CHARACTER_GEN_RAM : label is "1010101010101010202020202020202040404040404040408080808080808080";
  attribute INIT_07 of CHARACTER_GEN_RAM : label is "0101010101010101020202020202020204040404040404040808080808080808";
  attribute INIT_08 of CHARACTER_GEN_RAM : label is "0028287C287C2828000000000028282800100000101010100000000000000000";
  attribute INIT_09 of CHARACTER_GEN_RAM : label is "0000000000201030003448542050483000064620100864600010781438503C10";
  attribute INIT_0A of CHARACTER_GEN_RAM : label is "000010107C101000000010543854100000201008080810200008102020201008";
  attribute INIT_0B of CHARACTER_GEN_RAM : label is "00004020100804000030300000000000000000007C0000002010300000000000";
  attribute INIT_0C of CHARACTER_GEN_RAM : label is "003844040810087C007C201008044438003810101010301000384464544C4438";
  attribute INIT_0D of CHARACTER_GEN_RAM : label is "002020201008047C0038444478402018003844040478407C0008087C48281808";
  attribute INIT_0E of CHARACTER_GEN_RAM : label is "20103000003030000030300000303000003008043C4444380038444438444438";
  attribute INIT_0F of CHARACTER_GEN_RAM : label is "001000100804443800402010081020200000007C007C00000008102040201008";

  attribute INIT_10 of CHARACTER_GEN_RAM : label is "003844404040443800784444784444780044447C444444380038545434044438";
  attribute INIT_11 of CHARACTER_GEN_RAM : label is "003C44445C404438004040407840407C007C40407840407C0070484444444870";
  attribute INIT_12 of CHARACTER_GEN_RAM : label is "0044485060504844003048080808081C0038101010101038004444447C444444";
  attribute INIT_13 of CHARACTER_GEN_RAM : label is "00384444444444380044444C546444440044444454546C44007C404040404040";
  attribute INIT_14 of CHARACTER_GEN_RAM : label is "007804043840403C004448507844447800344854444444380040404078444478";
  attribute INIT_15 of CHARACTER_GEN_RAM : label is "002854545444444400102844444444440038444444444444001010101010107C";
  attribute INIT_16 of CHARACTER_GEN_RAM : label is "0038202020202038007C40201008047C00101010284444440044442810284444";
  attribute INIT_17 of CHARACTER_GEN_RAM : label is "007C000000000000000000000044281000380808080808380000040810204000";
  attribute INIT_18 of CHARACTER_GEN_RAM : label is "00384040403800000078444464584040003C443C043800000000000000081020";
  attribute INIT_19 of CHARACTER_GEN_RAM : label is "38043C44443C000000202020702024180038407C44380000003C44444C340404";
  attribute INIT_1A of CHARACTER_GEN_RAM : label is "0048506050484040304808081800080000381010103000100044444464584040";
  attribute INIT_1B of CHARACTER_GEN_RAM : label is "0038444444380000004444446458000000444454546800000038101010101030";
  attribute INIT_1C of CHARACTER_GEN_RAM : label is "0078043840380000004040406458000004043C4C340000004040407844780000";
  attribute INIT_1D of CHARACTER_GEN_RAM : label is "0028545444440000001028444444000000344C44444400001824202070202000";
  attribute INIT_1E of CHARACTER_GEN_RAM : label is "0008101020101008007C2010087C000038043C44444400000044281028440000";
  attribute INIT_1F of CHARACTER_GEN_RAM : label is "AA55AA55AA55AA550000000C9260000000201010081010200010101010101010";

  attribute INIT_20 of CHARACTER_GEN_RAM : label is "111119694BEA5E00816123361830608000000000000000000000000000000000";
  attribute INIT_21 of CHARACTER_GEN_RAM : label is "0000000000000000000000000000000000000000000000000000000000000000";
  attribute INIT_22 of CHARACTER_GEN_RAM : label is "0000000000000000000000000000000000000000000000000000000000000000";
  attribute INIT_23 of CHARACTER_GEN_RAM : label is "0000000000000000000000000000000000000000000000000000000000000000";
  attribute INIT_24 of CHARACTER_GEN_RAM : label is "0000000000000000000000000000000000000000000000000000000000000000";
  attribute INIT_25 of CHARACTER_GEN_RAM : label is "0000000000000000000000000000000000000000000000000000000000000000";
  attribute INIT_26 of CHARACTER_GEN_RAM : label is "0000000000000000000000000000000000000000000000000000000000000000";
  attribute INIT_27 of CHARACTER_GEN_RAM : label is "0000000000000000000000000000000000000000000000000000000000000000";
  attribute INIT_28 of CHARACTER_GEN_RAM : label is "0000000000000000000000000000000000000000000000000000000000000000";
  attribute INIT_29 of CHARACTER_GEN_RAM : label is "0000000000000000000000000000000000000000000000000000000000000000";
  attribute INIT_2A of CHARACTER_GEN_RAM : label is "0000000000000000000000000000000000000000000000000000000000000000";
  attribute INIT_2B of CHARACTER_GEN_RAM : label is "0000000000000000000000000000000000000000000000000000000000000000";
  attribute INIT_2C of CHARACTER_GEN_RAM : label is "0000000000000000000000000000000000000000000000000000000000000000";
  attribute INIT_2D of CHARACTER_GEN_RAM : label is "0000000000000000000000000000000000000000000000000000000000000000";
  attribute INIT_2E of CHARACTER_GEN_RAM : label is "0000000000000000000000000000000000000000000000000000000000000000";
  attribute INIT_2F of CHARACTER_GEN_RAM : label is "0000000000000000000000000000000000000000000000000000000000000000";

  attribute INIT_30 of CHARACTER_GEN_RAM : label is "0000000000000000000000000000000000000000000000000000000000000000";
  attribute INIT_31 of CHARACTER_GEN_RAM : label is "0000000000000000000000000000000000000000000000000000000000000000";
  attribute INIT_32 of CHARACTER_GEN_RAM : label is "0000000000000000000000000000000000000000000000000000000000000000";
  attribute INIT_33 of CHARACTER_GEN_RAM : label is "0000000000000000000000000000000000000000000000000000000000000000";
  attribute INIT_34 of CHARACTER_GEN_RAM : label is "0000000000000000000000000000000000000000000000000000000000000000";
  attribute INIT_35 of CHARACTER_GEN_RAM : label is "0000000000000000000000000000000000000000000000000000000000000000";
  attribute INIT_36 of CHARACTER_GEN_RAM : label is "0000000000000000000000000000000000000000000000000000000000000000";
  attribute INIT_37 of CHARACTER_GEN_RAM : label is "0000000000000000000000000000000000000000000000000000000000000000";
  attribute INIT_38 of CHARACTER_GEN_RAM : label is "0000000000000000000000000000000000000000000000000000000000000000";
  attribute INIT_39 of CHARACTER_GEN_RAM : label is "0000000000000000000000000000000000000000000000000000000000000000";
  attribute INIT_3A of CHARACTER_GEN_RAM : label is "0000000000000000000000000000000000000000000000000000000000000000";
  attribute INIT_3B of CHARACTER_GEN_RAM : label is "0000000000000000000000000000000000000000000000000000000000000000";
  attribute INIT_3C of CHARACTER_GEN_RAM : label is "0000000000000000000000000000000000000000000000000000000000000000";
  attribute INIT_3D of CHARACTER_GEN_RAM : label is "0000000000000000000000000000000000000000000000000000000000000000";
  attribute INIT_3E of CHARACTER_GEN_RAM : label is "0000000000000000000000000000000000000000000000000000000000000000";
  attribute INIT_3F of CHARACTER_GEN_RAM : label is "0000000000000000000000000000000000000000000000000000000000000000";

  signal null_8  : std_logic_vector(7 downto 0);
  signal null_1  : std_logic_vector(0 downto 0);
  
begin  -- architecture IMP

  null_8 <= (others => '0');
  null_1 <= (others => '0');
  
  CHARACTER_GEN_RAM : RAMB16_S9_S9
    port map (
      -- OPB Port
      CLKA  => Clk,                     -- [in  std_ulogic]
      ENA   => '1',                     -- [in  std_ulogic]
      WEA   => Write,                   -- [in  std_ulogic]
      ADDRA => Address,                 -- [in  std_logic_vector (10 downto 0)]
      DIA   => Write_Data,              -- [in  std_logic_vector (7 downto 0)]
      DIPB  => null_1,                  -- [in  std_logic_vector (0 downto 0)]
      DOA   => Read_Data,               -- [out std_logic_vector (7 downto 0)]
      DOPA  => open,                    -- [out std_logic_vector (0 downto 0)]
      SSRA  => '0',                     -- [in  std_ulogic]

      -- Pixel Port 
      CLKB  => Pixel_Clock,             -- [in  std_ulogic]
      ENB   => '1',                     -- [in  std_ulogic]
      WEB   => '0',                     -- [in  std_ulogic]
      ADDRB => Pixel_Address,           -- [in  std_logic_vector (10 downto 0)]
      DIB   => null_8,                  -- [in  std_logic_vector (7 downto 0)]
      DIPA  => null_1,                  -- [in  std_logic_vector (0 downto 0)]
      DOB   => Pixel_Data,              -- [out std_logic_vector (7 downto 0)]
      DOPB  => open,                    -- [out std_logic_vector (0 downto 0)]
      SSRB  => '0');                    -- [in  std_ulogic]

end architecture IMP;
