------------------------------------------------------------------------------
-- $Id$
-------------------------------------------------------------------------------
-- character_mode.vhd
-------------------------------------------------------------------------------
--
--                  ****************************
--                  ** Copyright Xilinx, Inc. **
--                  ** All rights reserved.   **
--                  ****************************
--
-------------------------------------------------------------------------------
-- Filename:        character_mode.vhd
--
-- Description: This module is used to create the serial pixel data for the character 
--              mode svga display. The pixel data is common to all eight bits to be
--              displayed as bright white.
--                  
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--              character_mode.vhd
--
-------------------------------------------------------------------------------
-- Author:          goran
-- Revision:        $Revision$
-- Date:            $Date$
--
-- History:
--   rickb  2001-07-17    Verilog Version
--   goran  2002-01-18    VHDL Version
--
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;

entity character_mode is
  port (
    -- OPB read and write port
    Clk             : in  std_logic;
    Reset           : in  std_logic;
    Write_Char      : in  std_logic;
    Write_ROM       : in  std_logic;
    Address         : in  std_logic_vector(12 downto 0);
    Read_Char       : out std_logic_vector(7 downto 0);
    Read_ROM        : out std_logic_vector(7 downto 0);
    Write_Data      : in  std_logic_vector(7 downto 0);
    -- Pixel read port
    Pixel_Clock     : in  std_logic;
    Char_Address    : in  std_logic_vector(13 downto 0);
    Char_Pixel      : in  std_logic_vector(2 downto 0);
    Char_Line_Count : in  std_logic_vector(2 downto 0);
    Char_Mode_Data  : out std_logic_vector(7 downto 0)
    );

end entity character_mode;

architecture IMP of character_mode is

  component char_ram is
    port (
      -- OPB read and write port
      Clk           : in  std_logic;
      Address       : in  std_logic_vector(12 downto 0);
      Read_Data     : out std_logic_vector(7 downto 0);
      Write_Data    : in  std_logic_vector(7 downto 0);
      Write         : in  std_logic;
      -- Pixel read port
      Pixel_Clock   : in  std_logic;
      Pixel_Address : in  std_logic_vector(12 downto 0);
      Pixel_Read    : in  std_logic;
      Pixel_Data    : out std_logic_vector(7 downto 0));
  end component char_ram;

  component char_gen_rom is
    port (
      -- OPB read and write port
      Clk           : in  std_logic;
      Address       : in  std_logic_vector(10 downto 0);
      Read_Data     : out std_logic_vector(7 downto 0);
      Write_Data    : in  std_logic_vector(7 downto 0);
      Write         : in  std_logic;
      -- Pixel read port
      Pixel_Clock   : in  std_logic;
      Pixel_Address : in  std_logic_vector(10 downto 0);
      Pixel_Data    : out std_logic_vector(7 downto 0));
  end component char_gen_rom;

  signal ascii_code           : std_logic_vector(7 downto 0);
  signal char_gen_rom_address : std_logic_vector(10 downto 0);

  signal char_gen_rom_data    : std_logic_vector(7 downto 0);

  signal latch_data     : std_logic;
  signal latch_low_data : std_logic;
  signal shift_high     : std_logic;
  signal shift_low      : std_logic;

  signal char_pix              : std_logic;
  signal latched_low_char_data : std_logic_vector(3 downto 0);
  signal latched_char_data     : std_logic_vector(7 downto 0);
  
begin  -- architecture IMP

  char_ram_I : char_ram
    port map (
      -- OPB read and write port
      Clk           => Clk,             -- [in  std_logic]
      Address       => Address,         -- [in  std_logic_vector(12 downto 0)]
      Read_Data     => Read_Char,       -- [out std_logic_vector(7 downto 0)]
      Write_Data    => Write_Data,      -- [in  std_logic_vector(7 downto 0)]
      Write         => Write_Char,      -- [in  std_logic]
      -- Pixel read port
      Pixel_Clock   => Pixel_Clock,     -- [in  std_logic]
      Pixel_Address => Char_Address(12 downto 0),  -- [in  std_logic_vector(12 downto 0)]
      Pixel_Read    => '1',             -- [in  std_logic]
      Pixel_Data    => ascii_code);     -- [out std_logic_vector(7 downto 0)]


  char_gen_rom_I : char_gen_rom
    port map (
      -- OPB read and write port
      Clk           => Clk,             -- [in  std_logic]
      Address       => Address(10 downto 0),         -- [in  std_logic_vector(10 downto 0)]
      Read_Data     => Read_ROM,        -- [out std_logic_vector(7 downto 0)]
      Write_Data    => Write_Data,      -- [in  std_logic_vector(7 downto 0)]
      Write         => Write_ROM,       -- [in  std_logic]
      -- Pixel read port
      Pixel_Clock   => Pixel_Clock,     -- [in  std_logic]
      Pixel_Address => char_gen_rom_address,  -- [in  std_logic_vector(10 downto 0)]
      Pixel_Data    => char_gen_rom_data);  -- [out std_logic_vector(7 downto 0)]

  char_gen_rom_address <= ascii_code & Char_Line_Count;

  -----------------------------------------------------------------------------
  -- 
  -----------------------------------------------------------------------------
  Latch_Data_Proc : process (Pixel_Clock, Reset) is
  begin  -- process Latch_Data_Proc
    if Reset = '1' then                 -- asynchronous reset (active high)
      latch_data <= '0';
    elsif Pixel_Clock'event and Pixel_Clock = '1' then  -- rising clock edge
      if (Char_Pixel = "110") then
        latch_data <= '1';
      elsif (Char_Pixel = "111") then
        latch_data <= '0';
      end if;
    end if;
  end process Latch_Data_Proc;

  -----------------------------------------------------------------------------
  -- 
  -----------------------------------------------------------------------------
  Latch_Low_Data_Proc : process (Pixel_Clock, Reset) is
  begin  -- process Latch_Low_Data_Proc
    if Reset = '1' then                 -- asynchronous reset (active high)
      latch_low_data <= '0';
    elsif Pixel_Clock'event and Pixel_Clock = '1' then  -- rising clock edge
      if (Char_Pixel = "010") then
        latch_low_data <= '1';
      elsif (Char_Pixel = "011") then
        latch_low_data <= '0';
      end if;
    end if;
  end process Latch_Low_Data_Proc;

  -----------------------------------------------------------------------------
  -- 
  -----------------------------------------------------------------------------
  Shift_high_Proc : process (Pixel_Clock, Reset) is
  begin  -- process Shift_high_Proc
    if Reset = '1' then                 -- asynchronous reset (active high)
      shift_high <= '1';
    elsif Pixel_Clock'event and Pixel_Clock = '1' then  -- rising clock edge
      if (Char_Pixel = "011") then
        shift_high <= '0';
      elsif (Char_Pixel = "111") then
        shift_high <= '1';
      end if;
    end if;
  end process Shift_high_Proc;

  -----------------------------------------------------------------------------
  -- 
  -----------------------------------------------------------------------------
  Shift_low_Proc : process (Pixel_Clock, Reset) is
  begin  -- process Shift_low_Proc
    if Reset = '1' then                 -- asynchronous reset (active low)
      shift_low <= '0';
    elsif Pixel_Clock'event and Pixel_Clock = '1' then  -- rising clock edge
      if (Char_Pixel = "011") then
        shift_low <= '1';
      elsif (Char_Pixel = "111") then
        shift_low <= '0';
      end if;
    end if;
  end process Shift_low_Proc;

  -----------------------------------------------------------------------------
  -- 
  -----------------------------------------------------------------------------
  Serializer_Proc : process (Pixel_Clock, Reset) is
  begin  -- process Serializer_Proc
    if Reset = '1' then                 -- asynchronous reset (active high)
      char_Pix              <= '0';
      latched_low_char_data <= (others => '0');
      latched_char_data     <= (others => '0');
    elsif Pixel_Clock'event and Pixel_Clock = '1' then  -- rising clock edge
      if (shift_high = '1') then
        char_pix                      <= latched_char_data(7);
        latched_char_data(7 downto 4) <= latched_char_data(6 downto 4) & latched_char_data(7);
        if (latch_low_data = '1') then
          latched_low_char_data(3 downto 0) <= latched_char_data(3 downto 0);
        else
          latched_low_char_data(3 downto 0) <= latched_low_char_data(3 downto 0);
        end if;
      elsif (shift_low = '1') then
        char_pix                          <= latched_low_char_data(3);
        latched_low_char_data(3 downto 0) <= latched_low_char_data(2 downto 0) & latched_low_char_data(3);
        if (latch_data = '1') then
          latched_char_data <= char_gen_rom_data;
        else
          latched_char_data <= latched_char_data;
        end if;
      end if;
    end if;
  end process Serializer_Proc;

  Char_Mode_Data <= (others => char_pix);
end architecture IMP;
