-------------------------------------------------------------------------------
-- $Id$
-------------------------------------------------------------------------------
-- color_mode_svga_ctrl.vhd
-------------------------------------------------------------------------------
--
--                  ****************************
--                  ** Copyright Xilinx, Inc. **
--                  ** All rights reserved.   **
--                  ****************************
--
-------------------------------------------------------------------------------
-- Filename:        color_mode_svga_ctrl.vhd
--
-- Description:     This is the top level module for the color character mode SVGA display.
--      This design supports screen resolutions up to 800 x 600 (75 rows of 100 characters).
--      This module uses 6 block RAMS to store the ASCII characters and the color definitions
--      and a block RAM as a character generator ROM. No external RAMS are required. 
--      3 additional block RAMS are required if the resolution is increased to 1024 x 768
--      the maximum supported by the SVGA_TIMING_GENERATION module.

--      The user has unrestricted access to the video memory because of the two port
--      behavior of the BRAMS.
--                  
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--              color_mode_svga_ctrl.vhd
--
-------------------------------------------------------------------------------
-- Author:          goran
-- Revision:        $Revision$
-- Date:            $Date$
--
-- History:
--   rickb  2001-07-17    Verilog Version
--   goran  2002-01-21    VHDL Version
--
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity color_mode_svga_ctrl is
  generic (
    C_VIDEO_MODE    : integer := 6;
    C_BM_MODE       : integer := 1;
    C_CHAR_MODE     : integer := 1
    );
  port (
    -- Processor signals
    OPB_Clk             : in  std_logic;
    OPB_Rst             : in  std_logic;
    Address             : in  std_logic_vector(12 downto 0);
    Write_Data          : in  std_logic_vector(23 downto 0);
    Read_Data           : out std_logic_vector(23 downto 0);
    Char_Mem_Access     : in  std_logic;  -- Access ascii and color information
    Write_Enable        : in  std_logic;  -- write enable
    Char_ROM_Access     : in  std_logic;  -- Access the character ROM
    BM_Display_Active   : in  std_logic;
    Char_Display_Active : in  std_logic;

    -- Video signals
    Pixel_Clock   : in  std_logic;      -- pixel clock input 
    Char_Delay    : in  std_logic_vector(0 to 2);

    -- Video out signals
    VGA_HSYNCH_N          : out std_logic;  -- horizontal sync for the VGA output connector
    VGA_VSYNCH_N          : out std_logic;  -- vertical sync for the VGA output connector
    VGA_OUT_RED_P         : out std_logic;  -- RED DAC data
    VGA_OUT_GREEN_P       : out std_logic;  -- GREEN DAC data
    VGA_OUT_BLUE_P        : out std_logic   -- BLUE DAC data
    ;
    Read_Pixel_dbg : out std_logic;
    Read_Data_dbg : out std_logic_vector(0 to 31)
    );

end entity color_mode_svga_ctrl;

library unisim;
use unisim.all;

architecture IMP of color_mode_svga_ctrl is

  component FDCPE is
    port (
      Q   : out std_logic;
      C   : in  std_logic;
      CE  : in  std_logic;
      D   : in  std_logic;
      PRE : in  std_logic;
      CLR : in  std_logic);
  end component FDCPE;
  
  component FIFO is
    generic (
      C_DEPTH : natural;                -- FIFO Data Depth
      C_WIDTH : natural);               -- Nr of bits needed for the data
    port (
      Reset_FIFO : in std_logic;

      Write_Clk    : in  std_logic;
      Write        : in  std_logic;
      Write_Data   : in  std_logic_vector(C_WIDTH-1 downto 0);
      Full         : out std_logic;
      Half_To_Full : out std_logic;

      Read_Clk      : in  std_logic;
      Read          : in  std_logic;
      Read_Data     : out std_logic_vector(C_WIDTH-1 downto 0);
      Empty         : out std_logic;
      Empty_to_Half : out std_logic);
  end component FIFO;

  component Drive_DAC_Data is
    port (
      bm_red_data         : in  std_logic;
      bm_green_data       : in  std_logic;
      bm_blue_data        : in  std_logic;
      char_red_data       : in  std_logic;
      char_green_data     : in  std_logic;
      char_blue_data      : in  std_logic;
      bm_display_active   : in  std_logic;
      char_display_active : in  std_logic;
      dac_red_data        : out std_logic;
      dac_green_data      : out std_logic;
      dac_blue_data       : out std_logic);
  end component Drive_DAC_Data;

  component character_mode is
    port (
      -- OPB read and write port
      Clk             : in  std_logic;
      Reset           : in  std_logic;
      Write_Char      : in  std_logic;
      Write_ROM       : in  std_logic;
      Address         : in  std_logic_vector(12 downto 0);
      Read_Char       : out std_logic_vector(7 downto 0);
      Read_ROM        : out std_logic_vector(7 downto 0);
      Write_Data      : in  std_logic_vector(7 downto 0);
      -- Pixel read port
      Pixel_Clock     : in  std_logic;
      Char_Address    : in  std_logic_vector(13 downto 0);
      Char_Pixel      : in  std_logic_vector(2 downto 0);
      Char_Line_Count : in  std_logic_vector(2 downto 0);
      Char_Mode_Data  : out std_logic_vector(7 downto 0));
  end component character_mode;

  component Color_RAM is
    port (
      -- Processor Interface
      Clk        : in  std_logic;
      Address    : in  std_logic_vector(12 downto 0);
      Write      : in  std_logic;
      Write_Data : in  std_logic_vector(3 downto 0);
      Read_Data  : out std_logic_vector(3 downto 0);

      -- Pixel Interface
      Pixel_Clock : in  std_logic;
      Color_Data  : out std_logic_vector(3 downto 0);
      Color_Addr  : in  std_logic_vector(12 downto 0);
      Color_Read  : in  std_logic);
  end component Color_RAM;

  component color_pipe is
    port (
      Pixel_Clock        : in  std_logic;
      Reset              : in  std_logic;
      Color_Ram_Data     : in  std_logic_vector(3 downto 0);
      Aligned_Color_Data : out std_logic_vector(3 downto 0));
  end component color_pipe;

  component SVGA_Timing_Generation is
    generic (
      C_VIDEO_MODE : integer := 6;
      C_BM_MODE    : integer := 1;
      C_CHAR_MODE  : integer := 1
      );
    port (
      Reset            : in  std_logic;
      Pixel_Clock      : in  std_logic;
      Char_Delay       : in  std_logic_vector(0 to 2);
      H_Synch_Delay    : out std_logic;
      V_Synch_Delay    : out std_logic;
      Comp_Synch       : out std_logic;
      Blank            : out std_logic;
      Consume_Pixel    : out std_logic;
      Reset_Pixel_Addr : out std_logic;
      Char_Line_Count  : out std_logic_vector(2 downto 0);
      Char_Address     : out std_logic_vector(13 downto 0);
      Char_Pixel       : out std_logic_vector(2 downto 0));
  end component SVGA_Timing_Generation;

  signal Char_Line_Count : std_logic_vector(2 downto 0);
  signal Char_Address    : std_logic_vector(13 downto 0);
  signal Char_Pixel      : std_logic_vector(2 downto 0);
  signal H_Synch_Delay   : std_logic;
  signal V_Synch_Delay   : std_logic;
  signal Comp_Synch      : std_logic;
  signal Blank           : std_logic;

  signal Char_Mode_Data : std_logic_vector(7 downto 0);

  signal Color_Data : std_logic_vector(3 downto 0);

  signal Aligned_Color_Data : std_logic_vector(3 downto 0);

  signal char_Red_Data   : std_logic;
  signal char_Green_Data : std_logic;
  signal char_Blue_Data  : std_logic;
  signal bm_Red_Data     : std_logic;
  signal bm_Green_Data   : std_logic;
  signal bm_Blue_Data    : std_logic;
  signal dac_red_data    : std_logic;
  signal dac_green_data  : std_logic;
  signal dac_blue_data   : std_logic;

  signal Write_ROM  : std_logic;
  signal Write_Char : std_logic;

  signal Read_Char_Data : std_logic_vector(11 downto 0);
  signal Read_ROM_Data  : std_logic_vector(7 downto 0);

  signal Reset_Pixel_Addr       : std_logic;
  signal Reset_Pixel_Addr_OPB_1 : std_logic;
  signal Reset_Pixel_Addr_OPB   : std_logic;
  signal FIFO_almost_full       : std_logic;
  signal FIFO_full              : std_logic;
  signal FIFO_empty             : std_logic;
  signal read_Pixel             : std_logic;
  signal consume_Pixel          : std_logic;
  signal Video_Adress_i         : std_logic_vector(19 downto 0);

  signal Pixel_Data : std_logic_vector(31 downto 0);
  
  
begin  -- architecture IMP
  
  Read_Handle : process ( Char_Mem_Access, Char_ROM_Access,
                          Read_Char_Data, Read_ROM_Data) is
    variable tmp : std_logic_vector(1 downto 0);
  begin  -- process Read_Handle
    Read_Data <= (others => '0');
    tmp       := Char_Mem_Access & Char_ROM_Access;
    case tmp is
      when "10"  => Read_Data(Read_Char_Data'range) <= Read_Char_Data;
      when "01"  => Read_Data(Read_ROM_Data'range)  <=  Read_ROM_Data;
      when others => Read_Data <= (others => '0');
    end case;
  end process Read_Handle;

  Blank_on_Reset : process (Pixel_Clock, OPB_Rst) is
  begin  -- process Blank_on_Reset
    if OPB_Rst = '1' then               -- asynchronous reset (active high)
      VGA_HSYNCH_N     <= '1';
      VGA_VSYNCH_N     <= '1';
      VGA_OUT_RED_P    <= '0';
      VGA_OUT_GREEN_P  <= '0';
      VGA_OUT_BLUE_P   <= '0';
    elsif Pixel_Clock'event and Pixel_Clock = '1' then  -- rising clock edge
      VGA_HSYNCH_N     <= not H_Synch_Delay;
      VGA_VSYNCH_N     <= not V_Synch_Delay;
      VGA_OUT_RED_P    <= dac_red_data;
      VGA_OUT_GREEN_P  <= dac_green_data;
      VGA_OUT_BLUE_P   <= dac_blue_data;
    end if;
  end process Blank_on_Reset;

  -- instance "SVGA_Timing_Generation_I"
  SVGA_Timing_Generation_I : SVGA_Timing_Generation
    generic map (
      C_VIDEO_MODE => C_VIDEO_MODE,     -- [integer]
      C_CHAR_MODE  => C_CHAR_MODE,      -- [integer]
      C_BM_MODE    => C_BM_MODE         -- [integer]
      )
    port map (
      Reset            => OPB_Rst,      -- [in  std_logic]
      Pixel_Clock      => Pixel_Clock,  -- [in  std_logic]
      Char_Delay       => Char_Delay,
      H_Synch_Delay    => H_Synch_Delay,    -- [out std_logic]
      V_Synch_Delay    => V_Synch_Delay,    -- [out std_logic]
      Comp_Synch       => Comp_Synch,   -- [out std_logic]
      Blank            => Blank,        -- [out std_logic]
      Consume_Pixel    => consume_Pixel,    -- [out std_logic]
      Reset_Pixel_Addr => reset_Pixel_Addr,  -- [out std_logic]
      Char_Line_Count  => Char_Line_Count,  -- [out std_logic_vector(2 downto 0)]
      Char_Address     => Char_Address,  -- [out std_logic_vector(13 downto 0)]
      Char_Pixel       => Char_Pixel);  -- [out std_logic_vector(2 downto 0)]

  Using_Char_Video : if (C_CHAR_MODE = 1) generate

    -- instance "character_mode_I"
    character_mode_I : character_mode
      port map (
        -- OPB read and write port
        Clk             => OPB_Clk,     -- [in  std_logic]
        Reset           => OPB_Rst,     -- [in  std_logic]
        Write_Char      => Write_Char,  -- [in  std_logic]
        Write_ROM       => Write_ROM,   -- [in  std_logic]
        Address         => Address(12 downto 0),  -- [in  std_logic_vector(12 downto 0)]
        Read_Char       => Read_Char_Data(7 downto 0),  -- [out std_logic_vector(7 downto 0)]
        Read_ROM        => Read_ROM_Data,  -- [out std_logic_vector(7 downto 0)]
        Write_Data      => Write_Data(7 downto 0),  -- [in  std_logic_vector(7 downto 0)]
        -- Pixel read port
        Pixel_Clock     => Pixel_Clock,   -- [in  std_logic]
        Char_Address    => Char_Address,  -- [in  std_logic_vector(13 downto 0)]
        Char_Pixel      => Char_Pixel,  -- [in  std_logic_vector(2 downto 0)]
        Char_Line_Count => Char_Line_Count,  -- [in  std_logic_vector(2 downto 0)]
        Char_Mode_Data  => Char_Mode_Data);  -- [out std_logic_vector(7 downto 0)]

    Write_Char <= Write_Enable and Char_Mem_Access;
    Write_ROM  <= Write_Enable and Char_ROM_Access;

    Color_RAM_I : Color_RAM
      port map (
        -- Processor Interface
        Clk        => OPB_Clk,          -- [in  std_logic]
        Address    => Address(12 downto 0),  -- [in  std_logic_vector(12 downto 0)]
        Write      => Write_Char,       -- [in  std_logic]
        Write_Data => Write_Data(11 downto 8),  -- [in  std_logic_vector(3 downto 0)]
        Read_Data  => Read_Char_Data(11 downto 8),  -- [out std_logic_vector(3 downto 0)]

        -- Pixel Interface
        Pixel_Clock => Pixel_Clock,     -- [in  std_logic]
        Color_Data  => Color_Data,      -- [out std_logic_vector(3 downto 0)]
        Color_Addr  => Char_Address(12 downto 0),  -- [in  std_logic_vector(12 downto 0)]
        Color_Read  => '1');            -- [in std_logic]

    -- instance "color_pipe_I"
    color_pipe_I : color_pipe
      port map (
        Pixel_Clock        => Pixel_Clock,          -- [in  std_logic]
        Reset              => OPB_Rst,  -- [in  std_logic]
        Color_Ram_Data     => Color_Data,  -- [in  std_logic_vector(3 downto 0)]
        Aligned_Color_Data => Aligned_Color_Data);  -- [out std_logic_vector(3 downto 0)]

    Out_DFF : process (Pixel_Clock) is
    begin  -- process Out_DFF
      if Pixel_Clock'event and Pixel_Clock = '1' then  -- rising clock edge
        if (OPB_Rst = '1') or (Char_Mode_Data(0) = '0') then
          Char_Red_Data   <= '0';
          Char_Green_Data <= '0';
          Char_Blue_Data  <= '0';
        else
          Char_Red_Data   <= Aligned_Color_Data(0);
          Char_Green_Data <= Aligned_Color_Data(1);
          Char_Blue_Data  <= Aligned_Color_Data(2);
        end if;
      end if;
    end process Out_DFF;    
  end generate Using_Char_Video;

  No_Char_Video : if (C_CHAR_MODE = 0) generate
--    Read_Char_Data  <= '0';
--    Read_ROM_Data   <= '0';
--    Read_CLUT_Data  <= '0';
    char_Red_Data   <= '0';
    char_Green_Data <= '0';
    char_Blue_Data  <= '0';
  end generate No_Char_Video;

  No_BM_Video : if (C_BM_MODE = 0) generate
    bm_Red_Data   <= '0';
    bm_Green_Data <= '0';
    bm_Blue_Data  <= '0';
  end generate No_BM_Video;

--  Read_Data_dbg <= "00000000" & dac_red_data & dac_green_data & dac_blue_data;
  
  
  Drive_DAC_Data_I : Drive_DAC_Data
    port map (
      bm_red_data         => bm_red_data,  -- [in  std_logic]
      bm_green_data       => bm_green_data,  -- [in  std_logic]
      bm_blue_data        => bm_blue_data,  -- [in  std_logic]
      char_red_data       => char_red_data,  -- [in  std_logic]
      char_green_data     => char_green_data,  -- [in  std_logic]
      char_blue_data      => char_blue_data,  -- [in  std_logic]
      bm_display_active   => bm_display_active,    -- [in  std_logic]
      char_display_active => char_display_active,  -- [in  std_logic]
      dac_red_data        => dac_red_data,  -- [out std_logic]
      dac_green_data      => dac_green_data,  -- [out std_logic]
      dac_blue_data       => dac_blue_data);  -- [out std_logic]

end architecture IMP;
