-------------------------------------------------------------------------------
-- $Id$
-------------------------------------------------------------------------------
-- drive_dac_data.vhd
-------------------------------------------------------------------------------
--
--                  ****************************
--                  ** Copyright Xilinx, Inc. **
--                  ** All rights reserved.   **
--                  ****************************
--
-------------------------------------------------------------------------------
-- Filename:        drive_dac_data.vhd
--
-- Description: This module enables either the bit-mapped image data from the 
--              Video RAMS, or the character-mapped text data from the 
--              character BRAMS to be passed to the Video DAC.
--                  
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--              drive_dac_data.vhd
--
-------------------------------------------------------------------------------
-- Author:          goran
-- Revision:        $Revision$
-- Date:            $Date$
--
-- History:
--   rickb  2001-07-17    Verilog Version
--   goran  2002-01-18    VHDL Version
--
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;

entity Drive_DAC_Data is
  port (
    bm_red_data         : in  std_logic;
    bm_green_data       : in  std_logic;
    bm_blue_data        : in  std_logic;
    char_red_data       : in  std_logic;
    char_green_data     : in  std_logic;
    char_blue_data      : in  std_logic;
    bm_display_active   : in  std_logic;
    char_display_active : in  std_logic;
    dac_red_data        : out std_logic;
    dac_green_data      : out std_logic;
    dac_blue_data       : out std_logic
    );

end entity Drive_DAC_Data;

architecture IMP of Drive_DAC_Data is

begin  -- architecture IMP

  Drive_DAC : process (bm_display_active, char_display_active,
                      char_red_data, char_green_data, char_blue_data,
                      bm_red_data, bm_green_data, bm_blue_data) is
    variable tmp : std_logic_vector(1 downto 0);
  begin  -- process Drive_DAC
    tmp := bm_display_active & char_display_active;
    case tmp is
      when "00" =>                      -- Nothing is active
        dac_red_data   <= '0';
        dac_green_data <= '0';
        dac_blue_data  <= '0';
      when "01" =>                      -- Character Mode is active
        dac_red_data   <= char_red_data;
        dac_green_data <= char_green_data;
        dac_blue_data  <= char_blue_data;
      when "10" =>                      -- Bit Mapped Mode is active
        dac_red_data   <= bm_red_data;
        dac_green_data <= bm_green_data;
        dac_blue_data  <= bm_blue_data;
      when "11" =>  -- Bit Mapped and Character Mode is active
        dac_red_data   <= bm_red_data or char_red_data;
        dac_green_data <= bm_green_data or char_green_data;
        dac_blue_data  <= bm_blue_data or char_blue_data;
      when others => null;
    end case;
  end process Drive_DAC;

end architecture IMP;
