-------------------------------------------------------------------------------
-- $Id$
-------------------------------------------------------------------------------
-- opb_color_video_ctrl.vhd
-------------------------------------------------------------------------------
--
--                  ****************************
--                  ** Copyright Xilinx, Inc. **
--                  ** All rights reserved.   **
--                  ****************************
--
-------------------------------------------------------------------------------
-- Filename:        opb_color_video_ctrl.vhd
--
-- Description:     
--                  
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--              opb_color_video_ctrl.vhd
--
-------------------------------------------------------------------------------
-- Author:          goran
-- Revision:        $Revision$
-- Date:            $Date$
--
-- History:
--   goran  2002-01-17    First Version
--
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use ieee.numeric_std.all;

entity opb_color_video_ctrl is
  generic (
    C_BASEADDR   : std_logic_vector(0 to 31) := X"FF700000";
    C_HIGHADDR   : std_logic_vector(0 to 31) := X"FF707FFF";
    C_VIDEO_MODE : integer                   := 6;
    C_BM_MODE    : integer                   := 0;
    C_CHAR_MODE  : integer                   := 0
    );
  port (
    OPB_Clk : in std_logic;
    OPB_Rst : in std_logic;

    -- OPB signals
    OPB_ABus    : in std_logic_vector(0 to 31);
    OPB_BE      : in std_logic_vector(0 to 3);
    OPB_RNW     : in std_logic;
    OPB_select  : in std_logic;
    OPB_seqAddr : in std_logic;
    OPB_DBus    : in std_logic_vector(0 to 31);

    Video_DBus    : out std_logic_vector(0 to 31);
    Video_errAck  : out std_logic;
    Video_retry   : out std_logic;
    Video_toutSup : out std_logic;
    Video_xferAck : out std_logic;

    -- Video signals
    Pixel_Clock     : in  std_logic;
    VGA_HSYNCH_N    : out std_logic;
    VGA_VSYNCH_N    : out std_logic;
    VGA_OUT_RED_P   : out std_logic;
    VGA_OUT_GREEN_P : out std_logic;
    VGA_OUT_BLUE_P  : out std_logic
    );  
end entity opb_color_video_ctrl;

library proc_common_v1_00_c;
use proc_common_v1_00_c.pselect;

library unisim;
use unisim.all;


architecture IMP of opb_color_video_ctrl is

  attribute BOX_TYPE : string;

  component pselect is
    generic (
      C_AB  : integer;
      C_AW  : integer;
      C_BAR : std_logic_vector);
    port (
      A      : in  std_logic_vector(0 to C_AW-1);
      AValid : in  std_logic;
      cs     : out std_logic);
  end component pselect;

  component color_mode_svga_ctrl is
    generic (
      C_VIDEO_MODE : integer := 6;
      C_BM_MODE    : integer := 1;
      C_CHAR_MODE  : integer := 1
      );
    port (
      -- Processor signals
      OPB_Clk             : in  std_logic;
      OPB_Rst             : in  std_logic;
      Address             : in  std_logic_vector(12 downto 0);
      Write_Data          : in  std_logic_vector(23 downto 0);
      Read_Data           : out std_logic_vector(23 downto 0);
      Char_Mem_Access     : in  std_logic;  -- Access ascii and color information
      Write_Enable        : in  std_logic;  -- write enable
      Char_ROM_Access     : in  std_logic;  -- Access the character ROM
      BM_Display_Active   : in  std_logic;
      Char_Display_Active : in  std_logic;

      -- Video signals
      Pixel_Clock   : in  std_logic;    -- pixel clock input
      Char_Delay    : in  std_logic_vector(0 to 2);

      -- Video out signals
      VGA_HSYNCH_N          : out std_logic;  -- horizontal sync for the VGA output connector
      VGA_VSYNCH_N          : out std_logic;  -- vertical sync for the VGA output connector
      VGA_OUT_RED_P         : out std_logic;  -- RED DAC data
      VGA_OUT_GREEN_P       : out std_logic;  -- GREEN DAC data
      VGA_OUT_BLUE_P        : out std_logic  -- BLUE DAC data
;
      Read_Pixel_dbg        : out std_logic;
      Read_Data_dbg         : out std_logic_vector(0 to 31)
      );
  end component color_mode_svga_ctrl;

  constant C_OPB_AWIDTH : natural := 32;
  
  function Addr_Bits (x, y : std_logic_vector(0 to C_OPB_AWIDTH-1)) return integer is
    variable addr_nor : std_logic_vector(0 to C_OPB_AWIDTH-1);
  begin
    addr_nor := x xor y;
    for i in 0 to C_OPB_AWIDTH-1 loop
      if addr_nor(i) = '1' then return i;
      end if;
    end loop;
    return(C_OPB_AWIDTH);
  end function Addr_Bits;

  constant C_AB : integer := Addr_Bits(C_HIGHADDR, C_BASEADDR);

  component FDRE is
    port (Q  : out std_logic;
          C  : in  std_logic;
          CE : in  std_logic;
          D  : in  std_logic;
          R  : in  std_logic);
  end component FDRE;

  component FDR is
    port (Q : out std_logic;
          C : in  std_logic;
          D : in  std_logic;
          R : in  std_logic);
  end component FDR;

  signal video_CS     : std_logic;
  signal video_CS_1   : std_logic;
  signal video_CS_2   : std_logic;
  signal video_CS_2_i : std_logic;
  signal video_CS_3   : std_logic;
  signal video_CS_3_i : std_logic;
  signal cs_2_i       : std_logic;

  signal opb_rnw_i : std_logic;

  signal null_16 : std_logic_vector(15 downto 0);
  signal null_2  : std_logic_vector(1 downto 0);
  signal null_1  : std_logic_vector(0 downto 0);

  signal video_xfer_Ack : std_logic;
  signal xfer_Ack_i     : std_logic;
  signal xfer_Ack       : std_logic;

  signal video_read_data : std_logic_vector(0 to 23);
  signal read_data       : std_logic_vector(0 to 31);

  signal OPB_ABus_i : std_logic_vector(16 to 18);

  signal write_enable     : std_logic;
  signal Char_Mem_Access  : std_logic;  -- Access ascii and color information
  signal Color_Map_Access : std_logic;  -- Access the color lookup table
  signal Char_ROM_Access  : std_logic;  -- Access the character ROM

  signal BM_Display_Active   : std_logic;
  signal Char_Display_Active : std_logic;


  signal VGA_VSYNCH_N_i : std_logic;

  signal Read_Pixel_dbg : std_logic;
  signal Read_Data_dbg  : std_logic_vector(0 to 31);

  signal Char_Delay : std_logic_vector(0 to 2);
  
begin  -- architecture IMP


  -----------------------------------------------------------------------------
  -- Handling the OPB bus interface
  -----------------------------------------------------------------------------

  video_CS_1_DFF : FDR
    port map (
      Q => video_CS_1,                  -- [out std_logic]
      C => OPB_Clk,                     -- [in  std_logic]
      D => video_CS,                    -- [in  std_logic]
      R => xfer_Ack);                   -- [in std_logic]

  Using_Character_Video : if (C_CHAR_MODE = 1) generate
    -- Do the OPB address decoding
    video_pselect_I : pselect
      generic map (
        C_AB  => C_AB,                  -- [integer]
        C_AW  => C_OPB_AWIDTH,          -- [integer]
        C_BAR => C_BASEADDR)            -- [std_logic_vector]
      port map (
        A      => OPB_ABus,             -- [in  std_logic_vector(0 to C_AW-1)]
        AValid => OPB_select,           -- [in  std_logic]
        cs     => video_CS);            -- [out std_logic]

    write_enable <= video_CS_2_i and not opb_rnw_i;

    video_CS_2_i <= not video_CS_3 and not video_CS_2 and video_CS_1;

    video_CS_2_DFF : FDR
      port map (
        Q => video_CS_2,                -- [out std_logic]
        C => OPB_Clk,                   -- [in  std_logic]
        D => video_CS_2_i,              -- [in  std_logic]
        R => xfer_Ack);                 -- [in std_logic]

    video_CS_3_i <= not video_CS_3 and video_CS_2 and video_CS_1;

    video_CS_3_DFF : FDR
      port map (
        Q => video_CS_3,                -- [out std_logic]
        C => OPB_Clk,                   -- [in  std_logic]
        D => video_CS_3_i,              -- [in  std_logic]
        R => xfer_Ack);                 -- [in std_logic]

  end generate Using_Character_Video;

  No_Char_Video : if (C_CHAR_MODE = 0) generate
    video_CS     <= '0';
    video_CS_2   <= '0';
    video_CS_2_i <= '0';
    video_CS_3   <= '0';
  end generate No_Char_Video;

  Video_errAck  <= '0';
  Video_retry   <= '0';
  Video_toutSup <= '0';

  -- Sampling OPB_RNW
  OPB_RNW_DFF : process (OPB_Clk, OPB_Rst)
  begin  -- process OPB_RNW_DFF
    if OPB_Rst = '1' then               -- asynchronous reset (active high)
      opb_rnw_i <= '0';
    elsif OPB_Clk'event and OPB_Clk = '1' then  -- rising clock edge
      opb_rnw_i <= OPB_RNW;
    end if;
  end process OPB_RNW_DFF;

  -- For Video Controll Access
  -- Xfer_Ack after 1 clock cycle for writes and after 3 clock cycles for read
  -- 
  video_xfer_Ack <= (video_CS_1 and not video_CS_2) when (opb_rnw_i = '0') else video_CS_3;

  Xfer_Ack_Handle : process (OPB_Clk, OPB_Rst) is
  begin  -- process Xfer_Ack_Handle
    if OPB_Rst = '1' then               -- asynchronous reset (active high)
      read_data <= (others => '0');
    elsif OPB_Clk'event and OPB_Clk = '1' then  -- rising clock edge
      read_data <= (others => '0');
      if (video_CS_1 = '1') then
        if (OPB_ABus_i(16 to 18) = "101") then
          read_data(31)       <= VGA_VSYNCH_N_i;
          read_data(30)       <= BM_Display_Active;
          read_data(29)       <= Char_Display_Active;
          read_data(28)       <= '1';
          read_data(27)       <= '1';
          read_data(24)       <= Read_Pixel_dbg;
          read_data(16 to 23) <= Read_Data_dbg(24 to 31);
          read_data(8 to 15)  <= (others => '0');
        else
          read_data(8 to 31) <= video_read_data;
        end if;
      end if;
    end if;
  end process Xfer_Ack_Handle;

  xfer_Ack_i <= video_xfer_Ack;

  xfer_Ack_DFF : FDR
    port map (
      Q => xfer_Ack,                    -- [out std_logic]
      C => OPB_Clk,                     -- [in  std_logic]
      D => xfer_Ack_i,                  -- [in  std_logic]
      R => xfer_Ack);                   -- [in std_logic]

  Video_xferAck <= xfer_Ack;

  cs_2_i <= video_CS_3_i;

  OPB_Read_Data_DFFs : for I in 0 to 31 generate
    FDRE_I : FDRE
      port map (
        Q  => Video_DBus(I),            -- [out std_logic]
        C  => OPB_Clk,                  -- [in  std_logic]
        CE => cs_2_i,                   -- [in  std_logic]
        D  => read_data(I),             -- [in  std_logic]
        R  => xfer_Ack);                -- [in std_logic]
  end generate OPB_Read_Data_DFFs;

  OPB_ABus_DFF : process (OPB_Clk, OPB_Rst) is
  begin  -- process OPB_ABus_DFF
    if OPB_Rst = '1' then               -- asynchronous reset (active high)
      OPB_ABus_i <= (others => '0');
    elsif OPB_Clk'event and OPB_Clk = '1' then  -- rising clock edge
      OPB_ABus_i <= OPB_ABus(OPB_ABus_i'range);
    end if;
  end process OPB_ABus_DFF;

  char_mem_access  <= '1' when video_CS_1 = '1' and OPB_ABus_i(16) = '0'         else '0';
  color_map_access <= '1' when video_CS_1 = '1' and OPB_ABus_i(16 to 18) = "100" else '0';
  char_rom_access  <= '1' when video_CS_1 = '1' and OPB_ABus_i(16 to 17) = "11"  else '0';

  Ctrl_Register : process (OPB_Clk, OPB_Rst) is
  begin  -- process Ctrl_Register
    if OPB_Rst = '1' then               -- asynchronous reset (active high)
      BM_Display_Active   <= '0';
      Char_Display_Active <= '0';
      Char_Delay          <= "101";     -- 5
--      Video_Bank          <= (others => '0');
    elsif OPB_Clk'event and OPB_Clk = '1' then  -- rising clock edge
      if (video_CS_1 = '1') and (OPB_ABus_i(16 to 18) = "101") and (opb_rnw_i = '0') then
        BM_Display_Active   <= OPB_DBus(31);
        Char_Display_Active <= OPB_DBus(30);
        Char_Delay          <= OPB_DBus(27 to 29);
--        Video_Bank          <= OPB_DBus(28 to 29);
      end if;
    end if;

  end process Ctrl_Register;

  ---------------------------------------------------------------------------
  -- Instanciate the video controller black box
  ---------------------------------------------------------------------------

  color_mode_svga_ctrl_I : color_mode_svga_ctrl
    generic map (
      C_VIDEO_MODE    => C_VIDEO_MODE,  -- [integer]
      C_BM_MODE       => C_BM_MODE,     -- [integer]
      C_CHAR_MODE     => C_CHAR_MODE    -- [integer]
      )
    port map (
      -- Processor signals
      OPB_Clk             => OPB_Clk,   -- [in  std_logic]
      OPB_Rst             => OPB_Rst,   -- [in  std_logic]
      Address             => OPB_ABus(17 to 29),  -- [in  std_logic_vector(12 downto 0)]
      Write_Data          => OPB_DBus(8 to 31),  -- [in  std_logic_vector(23 downto 0)]
      Read_Data           => Video_Read_Data(0 to 23),  -- [out std_logic_vector(23 downto 0)]
      Char_Mem_Access     => Char_Mem_Access,  -- [in  std_logic] Access ascii and color information
      Write_Enable        => Write_Enable,     -- [in  std_logic] write enable
      Char_ROM_Access     => Char_ROM_Access,  -- [in  std_logic] Access the character ROM
      BM_Display_Active   => BM_Display_Active,  -- [in  std_logic]
      Char_Display_Active => Char_Display_Active,       -- [in  std_logic]

      -- Video signals
      Pixel_Clock => Pixel_Clock,       -- [in  std_logic] pixel clock input
      Char_Delay  => Char_Delay,

      -- Video out signals
      VGA_HSYNCH_N          => VGA_HSYNCH_N,  -- [out std_logic] horizontal sync for the VGA output connector
      VGA_VSYNCH_N          => VGA_VSYNCH_N_i,  -- [out std_logic] vertical sync for the VGA output connector
      VGA_OUT_RED_P         => VGA_OUT_RED_P,  -- [out std_logic] RED DAC data
      VGA_OUT_GREEN_P       => VGA_OUT_GREEN_P,  -- [out std_logic] GREEN DAC data
      VGA_OUT_BLUE_P        => VGA_OUT_BLUE_P  -- [out std_logic] BLUE DAC data
      ,
      Read_Pixel_dbg        => Read_Pixel_dbg,
      Read_Data_dbg         => Read_Data_dbg
      );

  VGA_VSYNCH_N <= VGA_VSYNCH_N_i;


end architecture IMP;
