-------------------------------------------------------------------------------
-- $Id$
-------------------------------------------------------------------------------
-- FIFO.vhd
-------------------------------------------------------------------------------
--
--                  ****************************
--                  ** Copyright Xilinx, Inc. **
--                  ** All rights reserved.   **
--                  ****************************
--
-------------------------------------------------------------------------------
-- Filename:        FIFO.vhd
--
-- Description:     
--                  
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--              FIFO.vhd
--
-------------------------------------------------------------------------------
-- Author:          goran
-- Revision:        $Revision$
-- Date:            $Date$
--
-- History:
--   goran  2001-06-08    First Version
--
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity FIFO is
  generic (
    C_DEPTH    : natural := 16;         -- FIFO Data Depth
    C_WIDTH    : natural := 32          -- Nr of bits needed for the data
    );
  port (
    Reset_FIFO : in std_logic;

    Write_Clk    : in  std_logic;
    Write        : in  std_logic;
    Write_Data   : in  std_logic_vector(C_WIDTH-1 downto 0);
    Full         : out std_logic;
    Half_To_Full : out std_logic;

    Read_Clk      : in  std_logic;
    Read          : in  std_logic;
    Read_Data     : out std_logic_vector(C_WIDTH-1 downto 0);
    Empty         : out std_logic;
    Empty_to_Half : out std_logic
    );
end entity FIFO;

library unisim;
use unisim.all;


architecture IMP of FIFO is

  component LUT_RAM is
    generic (
      INIT_S : string);
    port (
      -- Processor Interface
      Clk        : in  std_logic;
      Addr       : in  std_logic_vector(3 downto 0);
      Write      : in  std_logic;
      Write_Data : in  std_logic;
      Read_Data  : out std_logic;
      Addr2      : in  std_logic_vector(3 downto 0);
      Read_Data2 : out std_logic);
  end component LUT_RAM;

  component FIFO_Count is
    port (
      Cnt_In  : in  std_logic_vector(3 downto 0);
      Cnt_Out : out std_logic_vector(3 downto 0));
  end component FIFO_Count;

  signal read_counter      : std_logic_vector(3 downto 0);
  signal write_counter     : std_logic_vector(3 downto 0);
  signal new_read_counter  : std_logic_vector(3 downto 0);
  signal new_write_counter : std_logic_vector(3 downto 0);

  signal set_direction   : std_logic;
  signal reset_direction : std_logic;
  signal direction       : std_logic;

  signal new_full  : std_logic;
  signal new_empty : std_logic;

  signal full_i  : std_logic;
  signal empty_i : std_logic;

  signal half_To_Full_i : std_logic;

  signal valid_read  : std_logic;
  signal valid_write : std_logic;

  signal starting : std_logic;

  signal equal : std_logic;

begin  -- architecture IMP

  -----------------------------------------------------------------------------
  -- Flag handling
  -----------------------------------------------------------------------------

  Start_From_Reset : process (Write_Clk, Reset_FIFO) is
  begin  -- process Start_From_Reset
    if Reset_FIFO = '1' then                 -- asynchronous reset (active high)
      starting <= '0';
    elsif Write_Clk'event and Write_Clk = '1' then  -- rising clock edge
      starting <= starting or write;
    end if;
  end process Start_From_Reset;

  -----------------------------------------------------------------------------
  --
  -- Divide the full counter range into four quadrants and assign the upper two
  -- values of the counter values to each quadrant. Movement in the quadrant is
  -- always clock-wise. Movement from one quadrant to the next is always grey
  -- coded (ie. only one bit change values). By tracking if one counter is just
  -- one quadrant behind or after the other counter, a reset or set condition
  -- to the direction signal can be generated. The direction signal is used for
  -- distinguish between full or empty when the counters are equal.
  -- 
  --                  |
  --               01 | 00        
  --              ---------
  --               11 | 10
  --                  |
  --
  --
  --
  --
  --
  -----------------------------------------------------------------------------
  Set_or_Reset_Direction : process (read_counter, write_counter) is
    variable Wr : std_logic_vector(1 downto 0);
    variable Rd : std_logic_vector(1 downto 0);
  begin  -- process Set_or_Reset_Direction
    Wr              := write_counter(3 downto 2);
    Rd              := read_counter(3 downto 2);
    reset_direction <= '0';
    set_direction   <= '0';
    if (Wr = "00") then
      if (Rd = "01") then
        reset_direction <= '1';
      elsif (Rd = "10") then
        set_direction <= '1';
      end if;
    elsif (Wr = "01") then
      if (Rd = "11") then
        reset_direction <= '1';
      elsif (Rd = "00") then
        set_direction <= '1';
      end if;
    elsif (Wr = "10") then
      if (Rd = "00") then
        reset_direction <= '1';
      elsif (Rd = "11") then
        set_direction <= '1';
      end if;
    elsif (Wr = "11") then
      if (Rd = "10") then
        reset_direction <= '1';
      elsif (Rd = "01") then
        set_direction <= '1';
      end if;
    end if;
  end process Set_or_Reset_Direction;

  Direction_Handle : process (direction, set_direction, reset_direction, starting) is
  begin  -- process Direction_Handle
    if (starting = '0') then
      direction <= '0';
    else
      direction <= not(reset_direction) and (set_direction or direction);
    end if;
  end process Direction_Handle;

  equal <= '1' when read_counter = write_counter else '0';

  new_full  <= equal and direction;
  new_empty <= equal and not direction;

  Full_Flag : process (Write_Clk, new_full) is
  begin  -- process Full_Flag
    if new_full = '1' then              -- asynchronous reset (active high)
      full_i <= '1';
    elsif Write_Clk'event and Write_Clk = '1' then  -- rising clock edge
      full_i <= new_full;
    end if;
  end process Full_Flag;

  Empty_Flag : process (Read_Clk, new_empty) is
  begin  -- process Empty_Flag
    if new_empty = '1' then             -- asynchronous reset (active high)
      empty_i <= '1';
    elsif Read_Clk'event and Read_Clk = '1' then  -- rising clock edge
      empty_i <= new_empty;
    end if;
  end process Empty_Flag;

  Half_Full_Handle : process (Write_Clk, Reset_FIFO) is
    variable Wr : std_logic_vector(3 downto 0);
    variable Rd : std_logic_vector(3 downto 0);
  begin  -- process Half_Full_Handle
    if Reset_FIFO = '1' then  -- asynchronous reset (active high)
      Half_To_Full_i <= '0';
    elsif Write_Clk'event and Write_Clk = '1' then  -- rising clock edge
      Half_To_Full_i <= '0';
      Wr := write_counter;
      Rd := read_counter;
      case rd is
        when "0000" =>
          if (Wr = "1111") or
            (Wr = "1110") or
            (Wr = "1100") or
            (Wr = "1101") or
            (Wr = "0101") or
            (Wr = "0111") or
            (Wr = "0110") or
            (Wr = "0100") then
            Half_To_Full_i <= '1';
          end if;
        when "0010" =>
          if
            (Wr = "1110") or
            (Wr = "1100") or
            (Wr = "1101") or
            (Wr = "0101") or
            (Wr = "0111") or
            (Wr = "0110") or
            (Wr = "0100") or
            (Wr = "0000")            
          then
            Half_To_Full_i <= '1';
          end if;
        when "0011" =>
          if
            (Wr = "1100") or
            (Wr = "1101") or
            (Wr = "0101") or
            (Wr = "0111") or
            (Wr = "0110") or
            (Wr = "0100") or
            (Wr = "0000") or
            (Wr = "0010")            
          then
            Half_To_Full_i <= '1';
          end if;
        when "0001" =>
          if
            (Wr = "1101") or
            (Wr = "0101") or
            (Wr = "0111") or
            (Wr = "0110") or
            (Wr = "0100") or
            (Wr = "0000") or
            (Wr = "0010") or           
            (Wr = "0011")            
          then
            Half_To_Full_i <= '1';
          end if;
        when "1001" =>
          if
            (Wr = "0101") or
            (Wr = "0111") or
            (Wr = "0110") or
            (Wr = "0100") or
            (Wr = "0000") or
            (Wr = "0010") or           
            (Wr = "0011") or           
            (Wr = "0001")           
          then
            Half_To_Full_i <= '1';
          end if;
        when "1000" =>
          if
            (Wr = "0111") or
            (Wr = "0110") or
            (Wr = "0100") or
            (Wr = "0000") or
            (Wr = "0010") or           
            (Wr = "0011") or           
            (Wr = "0001") or          
            (Wr = "1001")           
          then
            Half_To_Full_i <= '1';
          end if;
        when "1010" =>
          if
            (Wr = "0110") or
            (Wr = "0100") or
            (Wr = "0000") or
            (Wr = "0010") or           
            (Wr = "0011") or           
            (Wr = "0001") or          
            (Wr = "1001") or          
            (Wr = "1000")           
          then
            Half_To_Full_i <= '1';
          end if;
        when "1011" =>
          if
            (Wr = "0100") or
            (Wr = "0000") or
            (Wr = "0010") or           
            (Wr = "0011") or           
            (Wr = "0001") or          
            (Wr = "1001") or          
            (Wr = "1000") or          
            (Wr = "1010")           
          then
            Half_To_Full_i <= '1';
          end if;
        when "1111" =>
          if
            (Wr = "0000") or
            (Wr = "0010") or           
            (Wr = "0011") or           
            (Wr = "0001") or          
            (Wr = "1001") or          
            (Wr = "1000") or          
            (Wr = "1010") or          
            (Wr = "1011")           
          then
            Half_To_Full_i <= '1';
          end if;
        when "1110" =>
          if
            (Wr = "0010") or           
            (Wr = "0011") or           
            (Wr = "0001") or          
            (Wr = "1001") or          
            (Wr = "1000") or          
            (Wr = "1010") or          
            (Wr = "1011") or          
            (Wr = "1111")           
          then
            Half_To_Full_i <= '1';
          end if;
        when "1100" =>
          if
            (Wr = "0011") or           
            (Wr = "0001") or          
            (Wr = "1001") or          
            (Wr = "1000") or          
            (Wr = "1010") or          
            (Wr = "1011") or          
            (Wr = "1111") or          
            (Wr = "1110")           
          then
            Half_To_Full_i <= '1';
          end if;
        when "1101" =>
          if
            (Wr = "0001") or          
            (Wr = "1001") or          
            (Wr = "1000") or          
            (Wr = "1010") or          
            (Wr = "1011") or          
            (Wr = "1111") or          
            (Wr = "1110") or          
            (Wr = "1100")           
          then
            Half_To_Full_i <= '1';
          end if;
        when "0101" =>
          if
            (Wr = "1001") or          
            (Wr = "1000") or          
            (Wr = "1010") or          
            (Wr = "1011") or          
            (Wr = "1111") or          
            (Wr = "1110") or          
            (Wr = "1100") or          
            (Wr = "1101")           
          then
            Half_To_Full_i <= '1';
          end if;
        when "0111" =>
          if
            (Wr = "1000") or          
            (Wr = "1010") or          
            (Wr = "1011") or          
            (Wr = "1111") or          
            (Wr = "1110") or          
            (Wr = "1100") or          
            (Wr = "1101") or          
            (Wr = "0101")           
          then
            Half_To_Full_i <= '1';
          end if;
        when "0110" =>
          if
            (Wr = "1010") or          
            (Wr = "1011") or          
            (Wr = "1111") or          
            (Wr = "1110") or          
            (Wr = "1100") or          
            (Wr = "1101") or          
            (Wr = "0101") or          
            (Wr = "0111")           
          then
            Half_To_Full_i <= '1';
          end if;
        when "0100" =>
          if
            (Wr = "1011") or          
            (Wr = "1111") or          
            (Wr = "1110") or          
            (Wr = "1100") or          
            (Wr = "1101") or          
            (Wr = "0101") or          
            (Wr = "0111") or          
            (Wr = "0111")           
          then
            Half_To_Full_i <= '1';
          end if;
        when others => null;
      end case;
    end if;
  end process Half_Full_Handle;

  Half_To_Full <= half_To_Full_i or full_i;
  
  Half_Empty_Handle: process (Read_Clk, Reset_FIFO) is
  begin  -- process Half_Empty_Handle
    if Reset_FIFO = '1' then                 -- asynchronous reset (active high)
      Empty_to_half <= '0';      
    elsif Read_Clk'event and Read_Clk = '1' then  -- rising clock edge
      Empty_to_half <= '0';      
      if (reset_direction = '1') or
        ((direction = '0') and
         (write_counter(3 downto 2) = read_counter(3 downto 2))) then
        Empty_to_half <= '1';
      end if;
    end if;
  end process Half_Empty_Handle;
  
  Full  <= full_i;
  Empty <= empty_i;

  valid_read  <= read and not empty_i;
  valid_write <= write and not full_i;

  -----------------------------------------------------------------------------
  -- Counters
  -----------------------------------------------------------------------------
  Write_FIFO_Count_I : FIFO_Count
    port map (
      Cnt_In  => write_counter,         -- [in  std_logic_vector(3 downto 0)]
      Cnt_Out => new_write_counter);    -- [out std_logic_vector(3 downto 0)]

  Read_FIFO_Count_I : FIFO_Count
    port map (
      Cnt_In  => read_counter,          -- [in  std_logic_vector(3 downto 0)]
      Cnt_Out => new_read_counter);     -- [out std_logic_vector(3 downto 0)]

  Read_Count_Handle : process (Read_Clk, Reset_FIFO) is
  begin  -- process Read_Count_Handle
    if Reset_FIFO = '1' then                 -- asynchronous reset (active high)
      read_counter <= (others => '0');
    elsif Read_Clk'event and Read_Clk = '1' then  -- rising clock edge
      if (valid_read = '1') then
        read_counter <= new_read_counter;
      end if;
    end if;
  end process Read_Count_Handle;

  Write_Count_Handle : process (Write_Clk, Reset_FIFO) is
  begin  -- process Write_Count_Handle
    if Reset_FIFO = '1' then                 -- asynchronous reset (active high)
      write_counter <= (others => '0');
    elsif Write_Clk'event and Write_Clk = '1' then  -- rising clock edge
      if (Valid_Write = '1') then
        write_counter <= new_write_counter;
      end if;
    end if;
  end process Write_Count_Handle;

  -----------------------------------------------------------------------------
  -- FIFO RAM
  -----------------------------------------------------------------------------
  FIFO_RAM : for I in 0 to C_WIDTH-1 generate
    LUT_RAM_I : LUT_RAM
      generic map (
        INIT_S => "0000")               -- [string]
      port map (
        -- Processor Interface
        Clk        => Write_Clk,        -- [in  std_logic]
        Addr       => write_counter,    -- [in  std_logic_vector(3 downto 0)]
        Write      => Valid_Write,      -- [in  std_logic]
        Write_Data => Write_Data(I),    -- [in  std_logic]
        Read_Data  => open,             -- [out std_logic]
        Addr2      => read_counter,     -- [in  std_logic_vector(3 downto 0)]
        Read_Data2 => Read_Data(I));    -- [out std_logic]
  end generate FIFO_RAM;
  
end architecture IMP;

