-------------------------------------------------------------------------------
-- $Id$
-------------------------------------------------------------------------------
-- svga_timing_generation.vhd
-------------------------------------------------------------------------------
--
--                  ****************************
--                  ** Copyright Xilinx, Inc. **
--                  ** All rights reserved.   **
--                  ****************************
--
-------------------------------------------------------------------------------
-- Filename:        svga_timing_generation.vhd
--
-- Description:     
--                  
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--              svga_timing_generation.vhd
--
-------------------------------------------------------------------------------
-- Author:          goran
-- Revision:        $Revision$
-- Date:            $Date$
--
-- History:
--   rickb  2001-07-17    Verilog Version
--   goran  2002-01-21    VHDL Version
--
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

library unisim;
use unisim.all;

entity SVGA_Timing_Generation is
  generic (
    C_VIDEO_MODE : integer := 6;
    C_BM_MODE    : integer := 1;
    C_CHAR_MODE  : integer := 1
    );
  port (
    Reset            : in  std_logic;
    Pixel_Clock      : in  std_logic;
    Char_Delay       : in  std_logic_vector(0 to 2);
    H_Synch_Delay    : out std_logic;
    V_Synch_Delay    : out std_logic;
    Comp_Synch       : out std_logic;
    Blank            : out std_logic;
    Consume_Pixel    : out std_logic;
    Reset_Pixel_Addr : out std_logic;
    Char_Line_Count  : out std_logic_vector( 2 downto 0);
    Char_Address     : out std_logic_vector(13 downto 0);
    Char_Pixel       : out std_logic_vector( 2 downto 0)
    );

end entity SVGA_Timing_Generation;

architecture IMP of SVGA_Timing_Generation is

  constant ZBT_PIPELINE_DELAY     : integer := 2;
  constant ZBT_INTERFACE_DELAY    : integer := 2;
--  constant CHARACTER_DECODE_DELAY : integer := 5;
  signal CHARACTER_DECODE_DELAY : natural;

  subtype MODES is integer range 0 to 12;
  type    CONST_ARRAY is array(MODES) of integer;

  -- Mode  0  640 X 480 @ 60Hz with a 25.175MHz pixel clock
  -- Mode  1  640 X 480 @ 72Hz with a 31.500MHz pixel clock
  -- Mode  2  640 X 480 @ 75Hz with a 31.500MHz pixel clock
  -- Mode  3  640 X 480 @ 85Hz with a 36.000MHz pixel clock
  -- Mode  4  800 X 600 @ 56Hz with a 38.100MHz pixel clock
  -- Mode  5  800 X 600 @ 60Hz with a 40.000MHz pixel clock
  -- Mode  6  800 X 600 @ 72Hz with a 50.000MHz pixel clock
  -- Mode  7  800 X 600 @ 75Hz with a 49.500MHz pixel clock -- Strange Pixel Clock
  -- Mode  8  800 X 600 @ 85Hz with a 56.250MHz pixel clock
  -- Mode  9 1024 X 768 @ 60Hz with a 65.000MHz pixel clock
  -- Mode 10 1024 X 768 @ 70Hz with a 75.000MHz pixel clock
  -- Mode 11 1024 X 768 @ 75Hz with a 78.750MHz pixel clock
  -- Mode 12 1024 X 768 @ 85Hz with a 94.500MHz pixel clock


  constant H_ACTIVE_TABLE : CONST_ARRAY :=
    ( 0 => 640, 1 => 640, 2 => 640, 3 => 640,
      4 => 800, 5 => 800, 6 => 800, 7 => 800, 8 => 800,
      9 => 1024, 10 => 1024, 11 => 1024, 12 => 1024);

  constant H_ACTIVE : integer := H_ACTIVE_TABLE(C_VIDEO_MODE);

  constant H_FRONT_PORCH_TABLE : CONST_ARRAY :=
    (0  => 16,
     1  => 24,
     2  => 16,
     3  => 32,
     4  => 32,
     5  => 40,
     6  => 56,
     7  => 16,
     8  => 32,
     9  => 24,
     10 => 24,
     11 => 16,
     12 => 48);

  constant H_FRONT_PORCH : integer := H_FRONT_PORCH_TABLE(C_VIDEO_MODE);

  constant H_SYNCH_TABLE : CONST_ARRAY :=
    (0  => 96,
     1  => 40,
     2  => 96,
     3  => 48,
     4  => 128,
     5  => 128,
     6  => 120,
     7  => 80,
     8  => 64,
     9  => 136,
     10 => 136,
     11 => 96,
     12 => 96);

  constant H_SYNCH : integer := H_SYNCH_TABLE(C_VIDEO_MODE);

  constant H_BACK_PORCH_TABLE : CONST_ARRAY :=
    ( 0  => 48,
      1  => 128,
      2  => 48,
      3  => 112,
      4  => 128,
      5  => 88,
      6  => 64,
      7  => 160,
      8  => 152,
      9  => 160,
      10 => 144,
      11 => 176,
      12 => 208);

  constant H_BACK_PORCH : integer := H_BACK_PORCH_TABLE(C_VIDEO_MODE);

  constant H_TOTAL_TABLE : CONST_ARRAY :=
    ( 0  => 800,
      1  => 832,
      2  => 800,
      3  => 832,
      4  => 1088,
      5  => 1056,
      6  => 1040,
      7  => 1056,
      8  => 1048,
      9  => 1344,
      10 => 1328,
      11 => 1312,
      12 => 1376);

  constant H_TOTAL : integer := H_TOTAL_TABLE(C_VIDEO_MODE);

  constant V_ACTIVE_TABLE : CONST_ARRAY :=
    ( 0  => 480,
      1  => 480,
      2  => 480,
      3  => 480,
      4  => 600,
      5  => 600,
      6  => 600,
      7  => 600,
      8  => 600,
      9  => 768,
      10 => 768,
      11 => 768,
      12 => 768);

  constant V_ACTIVE : integer := V_ACTIVE_TABLE(C_VIDEO_MODE);

  constant V_FRONT_PORCH_TABLE : CONST_ARRAY :=
    ( 0  => 11,
      1  => 9,
      2  => 11,
      3  => 1,
      4  => 1,
      5  => 1,
      6  => 37,
      7  => 1,
      8  => 1,
      9  => 3,
      10 => 3,
      11 => 1,
      12 => 1 );

  constant V_FRONT_PORCH : integer := V_FRONT_PORCH_TABLE(C_VIDEO_MODE);

  constant V_SYNCH_TABLE : CONST_ARRAY :=
    ( 0  => 2,
      1  => 3,
      2  => 2,
      3  => 3,
      4  => 4,
      5  => 4,
      6  => 6,
      7  => 2,
      8  => 3,
      9  => 6,
      10 => 6,
      11 => 3,
      12 => 3);

  constant V_SYNCH : integer := V_SYNCH_TABLE(C_VIDEO_MODE);

  constant V_BACK_PORCH_TABLE : CONST_ARRAY :=
    ( 0  => 31,
      1  => 28,
      2  => 32,
      3  => 25,
      4  => 14,
      5  => 23,
      6  => 23,
      7  => 21,
      8  => 27,
      9  => 29,
      10 => 29,
      11 => 28,
      12 => 36 );

  constant V_BACK_PORCH : integer := V_BACK_PORCH_TABLE(C_VIDEO_MODE);


  constant V_TOTAL_TABLE : CONST_ARRAY :=
    ( 0  => 524,
      1  => 520,
      2  => 525,
      3  => 509,
      4  => 619,
      5  => 628,
      6  => 666,
      7  => 624,
      8  => 631,
      9  => 806,
      10 => 806,
      11 => 800,
      12 => 808 );

  constant V_TOTAL : integer := V_TOTAL_TABLE(C_VIDEO_MODE);

  signal pixel_count : natural range 0 to H_TOTAL - 1;
  signal line_count  : natural range 0 to V_TOTAL - 1;

  signal h_synch_i      : std_logic;
  signal v_synch_i      : std_logic;
  signal h_synch_delay0 : std_logic;
  signal v_synch_delay0 : std_logic;

  signal char_line_count_i  : std_logic_vector(2 downto 0);
  signal char_count         : std_logic_vector(16 downto 0);
  signal line_start_address : std_logic_vector(16 downto 0);
  signal char_pixel_i       : std_logic_vector(2 downto 0);
  
  signal reset_char_count : boolean;
  signal hold_char_count  : boolean;

  signal vga_ram_read_address       : std_logic_vector(19 downto 0);
  signal reset_vga_ram_read_address : boolean;
  signal line_hold_vga_ram_read_address  : boolean;
  signal vert_hold_vga_ram_read_address  : boolean;
  
begin  -- architecture IMP

--  CHARACTER_DECODE_DELAY <= to_integer(unsigned(Char_Delay));
  CHARACTER_DECODE_DELAY <= 2;
  
-- CREATE THE HORIZONTAL LINE PIXEL COUNTER
  pixel_count_count : process (Pixel_Clock, Reset) is
  begin
    if (reset = '1') then
      pixel_count <= 0;
    elsif Pixel_Clock'event and Pixel_Clock = '1' then
      if (pixel_count = (H_TOTAL - 1)) then
        pixel_count <= 0;               -- reset pixel counter
      else
        pixel_count <= pixel_count + 1;
      end if;
    end if;
  end process pixel_count_count;

-- CREATE THE HORIZONTAL SYNCH PULSE
  H_Synch_Handle : process (Pixel_Clock, Reset) is
  begin
    if (reset = '1') then
      h_synch_i <= '0';                 -- remove h_synch
    elsif Pixel_Clock'event and Pixel_Clock = '1' then
      if (pixel_count = (H_ACTIVE + H_FRONT_PORCH -1)) then
        h_synch_i <= '1';               -- start of h_synch
      elsif (pixel_count = (H_TOTAL - H_BACK_PORCH -1)) then
        h_synch_i <= '0';               -- end of h_synch
      end if;
    end if;
  end process H_Synch_Handle;


-- CREATE THE VERTICAL FRAME LINE COUNTER
  Line_Count_Handle : process (Pixel_Clock, Reset) is
  begin
    if (reset = '1') then
      line_count <= 0;
    elsif Pixel_Clock'event and Pixel_Clock = '1' then
      if ((line_count = (V_TOTAL - 1)) and (pixel_count = (H_TOTAL - 1))) then
                                        -- last pixel in last line of frame 
        line_count <= 0;                -- reset line counter
      elsif ((pixel_count = (H_TOTAL - 1))) then
                                        -- last pixel but not last line
        line_count <= line_count + 1;   -- increment line counter
      end if;
    end if;
  end process Line_Count_Handle;

-- CREATE THE VERTICAL SYNCH PULSE
  V_Synch_Handle : process (Pixel_Clock, Reset) is
  begin
    if (reset = '1') then
      v_synch_i <= '0';                           -- remove v_synch
    elsif Pixel_Clock'event and Pixel_Clock = '1' then
      if ((line_count = (V_ACTIVE + V_FRONT_PORCH -1) and
           (pixel_count = H_TOTAL - 1))) then     -- start of v_synch
        v_synch_i <= '1';
      elsif ((line_count = (V_TOTAL - V_BACK_PORCH - 1)) and
             (pixel_count = (H_TOTAL - 1))) then  -- end of v_synch
        v_synch_i <= '0';
      end if;
    end if;
  end process V_Synch_Handle;

-- ADD TWO PIPELINE DELAYS TO THE SYNCHs COMPENSATE FOR THE DAC PIPELINE DELAY
  DAC_Delays_Handle : process (Pixel_Clock, Reset) is
  begin
    if (reset = '1') then
      h_synch_delay0 <= '0';
      v_synch_delay0 <= '0';
      h_synch_delay  <= '0';
      v_synch_delay  <= '0';
    elsif Pixel_Clock'event and Pixel_Clock = '1' then
      h_synch_delay0 <= h_synch_i;
      v_synch_delay0 <= v_synch_i;
      h_synch_delay  <= h_synch_delay0;
      v_synch_delay  <= v_synch_delay0;
    end if;
  end process DAC_Delays_Handle;


  Using_Char_Video : if (C_CHAR_MODE = 1) generate

--    CREATE THE CHARACTER COUNTER.
--    CHARACTERS ARE DEFINED WITHIN AN 8 x 8 PIXEL BLOCK.

--      A 640  x 480 video mode will display 80  characters on 60 lines.
--      A 800  x 600 video mode will display 100 characters on 75 lines.
--      A 1024 x 768 video mode will display 128 characters on 96 lines.

--      "char_line_count" identifies the row in the 8 x 8 block.
--      "char_pixel" identifies the column in the 8 x 8 block.
--      "char_address" identifies the character in the total diaplay "0"
--      is the top left character.


-- CREATE THE VERTICAL FRAME LINE COUNTER
    Char_Line_Counter : process (Pixel_Clock, Reset) is
    begin
      if (reset = '1') then
        char_line_count_i <= "000";
      elsif Pixel_Clock'event and Pixel_Clock = '1' then
        if ((line_count = (V_TOTAL - 1)) and
            (pixel_count = (H_TOTAL - 1) - CHARACTER_DECODE_DELAY)) then
          char_line_count_i <= "000";   -- reset line counter
        elsif (pixel_count = (H_TOTAL - 1) - CHARACTER_DECODE_DELAY) then
          char_line_count_i <= std_logic_vector(to_unsigned(line_count, 3) + 1);  -- increment line counter
        end if;
      end if;
    end process Char_Line_Counter;

    Char_Line_Count <= char_line_count_i;

    Char_Counter : process (Pixel_Clock, Reset) is
    begin
      if (reset = '1') then
        char_count         <= (others => '0');
        line_start_address <= (others => '0');
      elsif Pixel_Clock'event and Pixel_Clock = '1' then
        if (reset_char_count) then      -- reset the char count during the VBI
          char_count         <= (others => '0');
          line_start_address <= (others => '0');
        elsif (not hold_char_count) then
          char_count <= std_logic_vector(unsigned(char_count) + 1);
        else  -- hold the character counter during the HBI
          if (char_line_count_i = "111") then  -- last line in the character block 
            line_start_address <= char_count;  -- update the line start address
          else
            -- not the last line in the char block
            char_count <= line_start_address;  -- restore the line start address
          end if;
        end if;
      end if;
    end process Char_Counter;

    char_address(13 downto 0) <= char_count(16 downto 3);

-- char_pixel defines the pixel within the character line
    Char_Pixel_Handle : process (Pixel_Clock, Reset) is
    begin
      if (reset = '1') then
        char_pixel_i <= "101";  -- reset to 5 so that the first character data can be latched
      elsif Pixel_Clock'event and Pixel_Clock = '1' then
        if (pixel_count = ((H_TOTAL - 1) - CHARACTER_DECODE_DELAY)) then
          char_pixel_i <= "101";  -- reset to 5 so that the first character data can be latched
        else
          char_pixel_i <= std_logic_vector(unsigned(char_pixel_i) + 1);
        end if;
      end if;
    end process Char_Pixel_Handle;

    Char_Pixel <= char_pixel_i;

-- CREATE THE CONTROL SIGNALS FOR THE CHARACTER ADDRESS COUNTER
-- The HOLD and RESET signals are advanced from the beginning and end
-- of HBI and VBI to compensate for the internal character generation
-- pipeline.

    Reset_Char_Count_Handle : process (Pixel_Clock, Reset) is
    begin
      if (reset = '1') then
        reset_char_count <= false;
      elsif Pixel_Clock'event and Pixel_Clock = '1' then
        if (line_count = (V_ACTIVE - 1) and
            pixel_count = ((H_ACTIVE - 1) - CHARACTER_DECODE_DELAY)) then  -- start of VBI
          reset_char_count <= true;
        elsif (line_count = (V_TOTAL - 1) and
               pixel_count = ((H_TOTAL - 1) - CHARACTER_DECODE_DELAY)) then  -- end of VBI 
          reset_char_count <= false;
        end if;
      end if;
    end process Reset_Char_Count_Handle;


    Hold_Char_Count_Handle : process (Pixel_Clock, Reset) is
    begin
      if (reset = '1') then
        hold_char_count <= false;
      elsif Pixel_Clock'event and Pixel_Clock = '1' then
        if (pixel_count = ((H_ACTIVE -1) - CHARACTER_DECODE_DELAY)) then  -- start of HBI
          hold_char_count <= true;
        elsif (pixel_count = ((H_TOTAL -1) - CHARACTER_DECODE_DELAY )) then  -- end of HBI
          hold_char_count <= false;
        end if;
      end if;
    end process Hold_Char_Count_Handle;

  end generate Using_Char_Video;

  No_Char_Video : if (C_CHAR_MODE = 0) generate
    Char_Line_Count <= (others => '0');
    Char_Address    <= (others => '0');
    Char_Pixel      <= (others => '0');
  end generate No_Char_Video;
  
  Using_BM_Video : if (C_BM_MODE = 1) generate

-- CREATE THE RAM ADDRESS COUNTER 
--     BM_Address_Counter : process (Pixel_Clock, Reset) is
--     begin
--       if (reset = '1') then
--         vga_ram_read_address <= (others => '0');
--       elsif Pixel_Clock'event and Pixel_Clock = '1' then  -- rising clock edge
--         if (reset_vga_ram_read_address) then
--           vga_ram_read_address <= (others => '0');
--         elsif (not hold_vga_ram_read_address) then
--           vga_ram_read_address <= std_logic_vector(unsigned(vga_ram_read_address) + 1);
--         end if;
--       end if;
--     end process BM_Address_Counter;

--     VGA_Ram_Address <= vga_ram_read_address;

-- CREATE THE CONTROL SIGNALS FOR THE VGA RAM ADDRESS COUNTER
    BM_Address_Reset : process (Pixel_Clock, Reset) is
    begin
      if (reset = '1') then
        reset_vga_ram_read_address <= false;
      elsif Pixel_Clock'event and Pixel_Clock = '1' then  -- rising clock edge
        if (line_count = V_ACTIVE) then
          reset_vga_ram_read_address <= true;  -- reset the address counter at the end of active video
        elsif (line_count = V_TOTAL - 2) then
          reset_vga_ram_read_address <= false;  -- re-enable the address counter at the start of active video
        end if;
      end if;
    end process BM_Address_Reset;

    Reset_Pixel_Addr <= '1' when reset_vga_ram_read_address else '0';
    
    BM_Address_Line_Hold : process (Pixel_Clock, Reset) is
    begin
      if (reset = '1') then
        line_hold_vga_ram_read_address <= false;
      elsif Pixel_Clock'event and Pixel_Clock = '1' then  -- rising clock edge
        if (pixel_count = (H_ACTIVE - 1 - 1)) then
          line_hold_vga_ram_read_address <= true;  -- hold the address counter at the end of active video
        elsif (pixel_count = (H_TOTAL - 1 - 1)) then
          line_hold_vga_ram_read_address <= false;  -- re-enable the address counter at the start of active video
        end if;
      end if;
    end process BM_Address_Line_Hold;
    
    BM_Address_Vert_Hold : process (Pixel_Clock, Reset) is
    begin
      if (reset = '1') then
        vert_hold_vga_ram_read_address <= false;
      elsif Pixel_Clock'event and Pixel_Clock = '1' then  -- rising clock edge
        if ((line_count = V_ACTIVE -1) and
            (pixel_count = (H_TOTAL - 1 - 2))) then
          vert_hold_vga_ram_read_address <= true;  -- hold the address counter at the end of active video
        elsif ((line_count = V_TOTAL -1) and
               (pixel_count = (H_TOTAL - 1 - 2))) then
          vert_hold_vga_ram_read_address <= false;  -- re-enable the address counter at the start of active video
        end if;
      end if;
    end process BM_Address_Vert_Hold;
    
    Consume_Pixel <= '0' when vert_hold_vga_ram_read_address or line_hold_vga_ram_read_address
                     else '1';

  end generate Using_BM_Video;

  No_BM_Video : if (C_BM_MODE = 0) generate
    Consume_Pixel    <= '0';
    Reset_Pixel_Addr <= '0';
  end generate No_BM_Video;

end architecture IMP;



